==> web2ldap-0.16.23.tar.gz: web2ldap-0.16.23/etc/web2ldap/web2ldapcnf/plugins/activedirectory.py <==
"""
web2ldap plugin classes for Active Directory (for some information see draft-armijo-ldap-syntax)

$Id: activedirectory.py,v 1.18 2008/01/15 11:00:52 michael Exp $
"""

import time,utctime,pyweblib,ldap.cidict,w2lapp.cnf,mspki.util

from w2lapp.gui import DisplayDN

from w2lapp.schema.syntaxes import \
  LDAPSyntax,Binary,Boolean,DistinguishedName,DirectoryString,GeneralizedTime, \
  Integer,BitArrayInteger,OctetString,SelectList,Uri,syntax_registry

class SAMAccountType(SelectList):
  """
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_samaccounttype.asp
  """
  oid = 'SAMAccountType-oid'
  desc = 'SAM-Account-Type in MS AD'
  attr_value_dict = {
     '268435456':'SAM_GROUP_OBJECT',
     '268435457':'SAM_NON_SECURITY_GROUP_OBJECT',
     '536870912':'SAM_ALIAS_OBJECT',
     '536870913':'SAM_NON_SECURITY_ALIAS_OBJECT',
     '805306368':'SAM_NORMAL_USER_ACCOUNT',
     '805306369':'SAM_MACHINE_ACCOUNT',
     '805306370':'SAM_TRUST_ACCOUNT',
    '1073741824':'SAM_APP_BASIC_GROUP',
    '1073741825':'SAM_APP_QUERY_GROUP',
    '2147483647':'SAM_ACCOUNT_TYPE_MAX',
  }

# Register certain attribute types for syntax classes
syntax_registry.registerAttrType(
  SAMAccountType.oid,[
    '1.2.840.113556.1.4.302', # sAMAccountType
  ]
)


class GroupType(SelectList):
  """
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_samaccounttype.asp
  """
  oid = 'GroupType-oid'
  desc = 'Group-Type in MS AD'
  attr_value_dict = {
    '-2147483640':'Universal / Security',
    '-2147483644':'Domain local / Security',
    '-2147483646':'Global / Security',
    '2':'Global / Distribution',
    '4':'Domain local / Distribution',
    '8':'Universal / Distribution',
  }

# Register certain attribute types for syntax classes
syntax_registry.registerAttrType(
  GroupType.oid,[
    '1.2.840.113556.1.4.750', # groupType
  ]
)


class UserAccountControl(BitArrayInteger):
  """
  See knowledge base article 305144:
  http://support.microsoft.com/default.aspx?scid=kb;en-us;Q305144
  """
  oid = 'UserAccountControl-oid'
  flag_desc_table = (
    ('SCRIPT',0x0001),
    ('ACCOUNTDISABLE',0x0002),
    ('HOMEDIR_REQUIRED',0x0008),
    ('LOCKOUT',0x0010),
    ('PASSWD_NOTREQD',0x0020),
    ('PASSWD_CANT_CHANGE',0x0040),
    ('ENCRYPTED_TEXT_PWD_ALLOWED',0x0080),
    ('TEMP_DUPLICATE_ACCOUNT',0x0100),
    ('NORMAL_ACCOUNT',0x0200),
    ('INTERDOMAIN_TRUST_ACCOUNT',0x0800),
    ('WORKSTATION_TRUST_ACCOUNT',0x1000),
    ('SERVER_TRUST_ACCOUNT',0x2000),
    ('DONT_EXPIRE_PASSWORD',0x10000),
    ('MNS_LOGON_ACCOUNT',0x20000),
    ('SMARTCARD_REQUIRED',0x40000),
    ('TRUSTED_FOR_DELEGATION',0x80000),
    ('NOT_DELEGATED',0x100000),
    ('USE_DES_KEY_ONLY',0x200000),
    ('DONT_REQ_PREAUTH',0x400000),
    ('PASSWORD_EXPIRED',0x800000),
    ('TRUSTED_TO_AUTH_FOR_DELEGATION',0x1000000)
  )

syntax_registry.registerAttrType(
  UserAccountControl.oid,[
    '1.2.840.113556.1.4.8', # userAccountControl
  ]
)


class SystemFlags(BitArrayInteger):
  """
  See
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_systemflags.asp
  and
  http://msdn2.microsoft.com/en-us/library/aa772297.aspx
  """
  oid = 'SystemFlags-oid'
  flag_desc_table = (
    ('ADS_SYSTEMFLAG_DISALLOW_DELETE',0x80000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_RENAME',0x40000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_MOVE',0x20000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_LIMITED_MOVE',0x10000000),
    ('ADS_SYSTEMFLAG_DOMAIN_DISALLOW_RENAME',0x08000000),
    ('ADS_SYSTEMFLAG_DOMAIN_DISALLOW_MOVE',0x04000000),
    ('ADS_SYSTEMFLAG_CR_NTDS_NC',0x00000001),
    ('ADS_SYSTEMFLAG_CR_NTDS_DOMAIN',0x00000002),
    ('ADS_SYSTEMFLAG_ATTR_NOT_REPLICATED',0x00000001),
    ('ADS_SYSTEMFLAG_ATTR_IS_CONSTRUCTED',0x00000004),
  )

syntax_registry.registerAttrType(
  SystemFlags.oid,[
    '1.2.840.113556.1.4.375', # systemFlags
  ]
)


class SearchFlags(BitArrayInteger):
  """
  """
  oid = 'SearchFlags-oid'
  flag_desc_table = (
    ('INDEXED',0x0001),
    ('',0x0002),
    ('',0x0004),
    ('',0x0008),
    ('',0x0010),
    ('',0x0020),
    ('CONFIDENTIAL',0x0040),
    ('',0x0080),
    ('',0x0100),
    ('',0x0200),
    ('',0x0400),
    ('',0x0800),
  )

syntax_registry.registerAttrType(
  SearchFlags.oid,[
    '1.2.840.113556.1.2.334', # searchFlags
  ]
)


class LogonHours(OctetString):
  oid = 'LogonHours-oid'
  desc = 'Logon hours'
  dayofweek = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat')

  def _extractHours(self,value):
    if not value or len(value)!=21:
      return []
    hour_flags = []
    for eight_hours in value:
      eight_hours_int = ord(eight_hours)
      for i in range(8):
        hour_flags.append({0:'-',1:'X'}[(eight_hours_int>>i)&1])
    # For whatever reason the list has to be shifted one hour
    return hour_flags

  def sanitizeInput(self,inputValue):
    inputValue = inputValue.replace('\r','').replace('\n','')
    hour_flags = [
      int(inputValue[i]=='X')<<i%8
      for i in xrange(len(inputValue))
    ]
    r = [
      chr(sum(hour_flags[i*8:(i+1)*8]))
      for i in xrange(21)
    ]
    attrValue = ''.join(r)
    return attrValue

  def validate(self,attrValue):
    return len(attrValue)==21

  def formField(self):
    hour_flags = self._extractHours(self.attrValue)
    if hour_flags:
      day_bits = [
        ''.join(hour_flags[24*day:24*day+24])
        for day in range(7)
      ]
    else:
      day_bits = []
    return pyweblib.forms.Textarea(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,1,
      None,
      default=u'\r\n'.join(day_bits),
      rows=7,cols=24
    )

  def displayValue(self,valueindex=0,commandbutton=0):
    hour_flags = self._extractHours(self.attrValue)
    result_lines = [
      """<tr>
      <th width="10%%">Day</th>
      <th colspan="3" width="8%%">0</th>
      <th colspan="3" width="8%%">3</th>
      <th colspan="3" width="8%%">6</th>
      <th colspan="3" width="8%%">9</th>
      <th colspan="3" width="8%%">12</th>
      <th colspan="3" width="8%%">15</th>
      <th colspan="3" width="8%%">18</th>
      <th colspan="3" width="8%%">21</th>
    </tr>""",
    ]
    for day in range(7):
      day_bits = hour_flags[24*day:24*day+24]
      result_lines.append(
        '<tr><td>%s</td><td>%s</td></tr>' % (
          self.dayofweek[day],
          '</td><td>'.join(day_bits)
      ))
    return '<p>%s</p><table border>%s</table>' % (
      OctetString.displayValue(self,valueindex,commandbutton),
      '\n'.join(result_lines)
    )


syntax_registry.registerAttrType(
  LogonHours.oid,[
    '1.2.840.113556.1.4.64', # logonHours
  ]
)


class CountryCode(SelectList):
  oid = 'CountryCode-oid'
  desc = 'Numerical country code'
  attr_value_dict = ldap.cidict.cidict(w2lapp.cnf.countries.countryCode_dict)


syntax_registry.registerAttrType(
  CountryCode.oid,[
    '1.2.840.113556.1.4.25', # countryCode
  ]
)


class InstanceType(BitArrayInteger):
  """
  http://msdn.microsoft.com/library/en-us/adschema/adschema/a_instancetype.asp
  """
  oid = 'InstanceType-oid'
  flag_desc_table = (
    ('The head of naming context.',0x00000001),
    ('This replica is not instantiated.',0x00000002),
    ('The object is writable on this directory.',0x00000004),
    ('The naming context above this one on this directory is held.',0x00000008),
    ('The naming context is in the process of being constructed for the first time via replication.',0x00000010),
    ('The naming context is in the process of being removed from the local DSA.',0x00000020),
  )

syntax_registry.registerAttrType(
  InstanceType.oid,[
    '1.2.840.113556.1.2.1', # instanceType
  ]
)


class DNWithOctetString(DistinguishedName):
  oid = '1.2.840.113556.1.4.903'
  desc = 'DNWithOctetString'

  def displayValue(self,valueindex=0,commandbutton=0):
    octet_tag,count,octet_string,dn = self.attrValue.split(':')
    return ':'.join([
      octet_tag,count,octet_string,
      DisplayDN(sid,form,ls,dn,commandbutton=commandbutton)
    ])


class DNWithString(DNWithOctetString):
  oid = '1.2.840.113556.1.4.904'
  desc = 'DNWithString'


class MicrosoftLargeInteger(Integer):
  oid = '1.2.840.113556.1.4.906'
  desc = 'Integer guaranteed to support 64 bit numbers'


class ObjectSecurityDescriptor(OctetString):
  oid = '1.2.840.113556.1.4.907'
  desc = 'Object-Security-Descriptor'


class Interval(MicrosoftLargeInteger):
  oid = 'Interval-oid'
  desc = 'Large integer with timestamp expressed as 100 nanoseconds since 1601-01-01 00:00'

  def displayValue(self,valueindex=0,commandbutton=0):
    if self.attrValue=='9223372036854775807':
      return '-1: unlimited/off'
    else:
      delta = (long(self.attrValue)-116444736000000000L)/10000000
      if delta>=0:
        return utctime.strftimeiso8601(time.gmtime(delta)).encode('ascii')
      else:
        return self.attrValue.encode('ascii')

# Register certain attribute types for Interval
syntax_registry.registerAttrType(
  Interval.oid,[
    '1.2.840.113556.1.4.159',  # accountExpires
    '1.2.840.113556.1.4.49',   # badPasswordTime
    '1.2.840.113556.1.4.52',   # lastLogon
    '1.2.840.113556.1.4.1696', # lastLogonTimestamp
    '1.2.840.113556.1.4.51',   # lastLogoff
    '1.2.840.113556.1.4.96',   # pwdLastSet
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))



# Work around for Active Directory of Windows 2000:
# Register syntaxes by odd names
syntax_registry.oid2syntax['Boolean'] = Boolean
syntax_registry.oid2syntax['DN'] = DistinguishedName
syntax_registry.oid2syntax['Integer'] = Integer
syntax_registry.oid2syntax['DirectoryString'] = DirectoryString
syntax_registry.oid2syntax['GeneralizedTime'] = GeneralizedTime

syntax_registry.registerAttrType(
  DistinguishedName.oid,[
    'configurationNamingContext',
    'defaultNamingContext',
    'dsServiceName',
    'rootDomainNamingContext',
    'schemaNamingContext',
  ]
)

# MS AD declares these attributes with OctetString
# syntax but Binary syntax is more suitable
syntax_registry.registerAttrType(
  Binary.oid,[
    '1.2.840.113556.1.4.645',    # userCert
    '1.2.840.113556.1.4.138',    # userParameters
  ]
)

syntax_registry.registerAttrType(
  Uri.oid,[
    '1.2.840.113556.1.4.583', # meetingURL
  ]
)

==> web2ldap-0.16.24.tar.gz: web2ldap-0.16.24/etc/web2ldap/web2ldapcnf/plugins/activedirectory.py <==
"""
web2ldap plugin classes for Active Directory (for some information see draft-armijo-ldap-syntax)

$Id: activedirectory.py,v 1.20 2008/06/11 09:52:11 michael Exp $
"""

import time,utctime,pyweblib,ldap.cidict,w2lapp.cnf,mspki.util

from w2lapp.gui import DisplayDN

from w2lapp.schema.syntaxes import \
  LDAPSyntax,Binary,Boolean,DistinguishedName,DirectoryString,GeneralizedTime, \
  Integer,BitArrayInteger,OctetString,SelectList,Uri,syntax_registry

class SAMAccountType(SelectList):
  """
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_samaccounttype.asp
  """
  oid = 'SAMAccountType-oid'
  desc = 'SAM-Account-Type in MS AD'
  attr_value_dict = {
     '268435456':'SAM_GROUP_OBJECT',
     '268435457':'SAM_NON_SECURITY_GROUP_OBJECT',
     '536870912':'SAM_ALIAS_OBJECT',
     '536870913':'SAM_NON_SECURITY_ALIAS_OBJECT',
     '805306368':'SAM_NORMAL_USER_ACCOUNT',
     '805306369':'SAM_MACHINE_ACCOUNT',
     '805306370':'SAM_TRUST_ACCOUNT',
    '1073741824':'SAM_APP_BASIC_GROUP',
    '1073741825':'SAM_APP_QUERY_GROUP',
    '2147483647':'SAM_ACCOUNT_TYPE_MAX',
  }

# Register certain attribute types for syntax classes
syntax_registry.registerAttrType(
  SAMAccountType.oid,[
    '1.2.840.113556.1.4.302', # sAMAccountType
  ]
)


class GroupType(SelectList):
  """
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_samaccounttype.asp
  """
  oid = 'GroupType-oid'
  desc = 'Group-Type in MS AD'
  attr_value_dict = {
    '-2147483640':'Universal / Security',
    '-2147483644':'Domain local / Security',
    '-2147483646':'Global / Security',
    '2':'Global / Distribution',
    '4':'Domain local / Distribution',
    '8':'Universal / Distribution',
  }

# Register certain attribute types for syntax classes
syntax_registry.registerAttrType(
  GroupType.oid,[
    '1.2.840.113556.1.4.750', # groupType
  ]
)


class UserAccountControl(BitArrayInteger):
  """
  See knowledge base article 305144:
  http://support.microsoft.com/default.aspx?scid=kb;en-us;Q305144
  """
  oid = 'UserAccountControl-oid'
  flag_desc_table = (
    ('SCRIPT',0x0001),
    ('ACCOUNTDISABLE',0x0002),
    ('HOMEDIR_REQUIRED',0x0008),
    ('LOCKOUT',0x0010),
    ('PASSWD_NOTREQD',0x0020),
    ('PASSWD_CANT_CHANGE',0x0040),
    ('ENCRYPTED_TEXT_PWD_ALLOWED',0x0080),
    ('TEMP_DUPLICATE_ACCOUNT',0x0100),
    ('NORMAL_ACCOUNT',0x0200),
    ('INTERDOMAIN_TRUST_ACCOUNT',0x0800),
    ('WORKSTATION_TRUST_ACCOUNT',0x1000),
    ('SERVER_TRUST_ACCOUNT',0x2000),
    ('DONT_EXPIRE_PASSWORD',0x10000),
    ('MNS_LOGON_ACCOUNT',0x20000),
    ('SMARTCARD_REQUIRED',0x40000),
    ('TRUSTED_FOR_DELEGATION',0x80000),
    ('NOT_DELEGATED',0x100000),
    ('USE_DES_KEY_ONLY',0x200000),
    ('DONT_REQ_PREAUTH',0x400000),
    ('PASSWORD_EXPIRED',0x800000),
    ('TRUSTED_TO_AUTH_FOR_DELEGATION',0x1000000)
  )

syntax_registry.registerAttrType(
  UserAccountControl.oid,[
    '1.2.840.113556.1.4.8', # userAccountControl
  ]
)


class SystemFlags(BitArrayInteger):
  """
  See
  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/adschema/adschema/a_systemflags.asp
  and
  http://msdn2.microsoft.com/en-us/library/aa772297.aspx
  """
  oid = 'SystemFlags-oid'
  flag_desc_table = (
    ('ADS_SYSTEMFLAG_DISALLOW_DELETE',0x80000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_RENAME',0x40000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_MOVE',0x20000000),
    ('ADS_SYSTEMFLAG_CONFIG_ALLOW_LIMITED_MOVE',0x10000000),
    ('ADS_SYSTEMFLAG_DOMAIN_DISALLOW_RENAME',0x08000000),
    ('ADS_SYSTEMFLAG_DOMAIN_DISALLOW_MOVE',0x04000000),
    ('ADS_SYSTEMFLAG_CR_NTDS_NC',0x00000001),
    ('ADS_SYSTEMFLAG_CR_NTDS_DOMAIN',0x00000002),
    ('ADS_SYSTEMFLAG_ATTR_NOT_REPLICATED',0x00000001),
    ('ADS_SYSTEMFLAG_ATTR_IS_CONSTRUCTED',0x00000004),
  )

syntax_registry.registerAttrType(
  SystemFlags.oid,[
    '1.2.840.113556.1.4.375', # systemFlags
  ]
)


class SearchFlags(BitArrayInteger):
  """
  http://msdn.microsoft.com/en-us/library/ms679765(VS.85).aspx

   1 (0x00000001) 	Create an index for the attribute.
   2 (0x00000002) 	Create an index for the attribute in each container.
   4 (0x00000004) 	Add this attribute to the Ambiguous Name Resolution (ANR) set. This is used to assist in finding an object when only partial information is given. For example, if the LDAP filter is (ANR=JEFF), the search will find each object where the first name, last name, e-mail address, or other ANR attribute is equal to JEFF. Bit 0 must be set for this index take affect.
   8 (0x00000008) 	Preserve this attribute in the tombstone object for deleted objects.
  16 (0x00000010) 	Copy the value for this attribute when the object is copied.
  32 (0x00000020) 	Supported beginning with Windows Server 2003. Create a tuple index for the attribute. This will improve searches where the wildcard appears at the front of the search string. For example, (sn=*mith).
  64 (0x00000040) 	Supported beginning with ADAM. Creates an index to greatly help VLV performance on arbitrary attributes.
  """
  oid = 'SearchFlags-oid'
  flag_desc_table = (
    ('Indexed',0x0001),
    ('Indexed in each container',0x0002),
    ('Ambiguous Name Resolution (ANR)',0x0004),
    ('Preserve in tombstone object',0x0008),
    ('Copy value when object copied',0x0010),
    ('tuple index',0x0020),
    ('VLV index (Subtree Index in ADAM)',0x0040),
    ('CONFIDENTIAL',0x0080),
    ('NEVER_AUDIT_VALUE',0x0100),
    ('RODC_FILTERED',0x0200),
    ('',0x0400),
    ('',0x0800),
  )

syntax_registry.registerAttrType(
  SearchFlags.oid,[
    '1.2.840.113556.1.2.334', # searchFlags
  ]
)


class LogonHours(OctetString):
  oid = 'LogonHours-oid'
  desc = 'Logon hours'
  dayofweek = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat')

  def _extractHours(self,value):
    if not value or len(value)!=21:
      return []
    hour_flags = []
    for eight_hours in value:
      eight_hours_int = ord(eight_hours)
      for i in range(8):
        hour_flags.append({0:'-',1:'X'}[(eight_hours_int>>i)&1])
    # For whatever reason the list has to be shifted one hour
    return hour_flags

  def sanitizeInput(self,inputValue):
    inputValue = inputValue.replace('\r','').replace('\n','')
    hour_flags = [
      int(inputValue[i]=='X')<<i%8
      for i in xrange(len(inputValue))
    ]
    r = [
      chr(sum(hour_flags[i*8:(i+1)*8]))
      for i in xrange(21)
    ]
    attrValue = ''.join(r)
    return attrValue

  def validate(self,attrValue):
    return len(attrValue)==21

  def formField(self):
    hour_flags = self._extractHours(self.attrValue)
    if hour_flags:
      day_bits = [
        ''.join(hour_flags[24*day:24*day+24])
        for day in range(7)
      ]
    else:
      day_bits = []
    return pyweblib.forms.Textarea(
      self.attrType,
      ': '.join([self.attrType,self.desc]),
      self.maxLen,1,
      None,
      default=u'\r\n'.join(day_bits),
      rows=7,cols=24
    )

  def displayValue(self,valueindex=0,commandbutton=0):
    hour_flags = self._extractHours(self.attrValue)
    result_lines = [
      """<tr>
      <th width="10%%">Day</th>
      <th colspan="3" width="8%%">0</th>
      <th colspan="3" width="8%%">3</th>
      <th colspan="3" width="8%%">6</th>
      <th colspan="3" width="8%%">9</th>
      <th colspan="3" width="8%%">12</th>
      <th colspan="3" width="8%%">15</th>
      <th colspan="3" width="8%%">18</th>
      <th colspan="3" width="8%%">21</th>
    </tr>""",
    ]
    for day in range(7):
      day_bits = hour_flags[24*day:24*day+24]
      result_lines.append(
        '<tr><td>%s</td><td>%s</td></tr>' % (
          self.dayofweek[day],
          '</td><td>'.join(day_bits)
      ))
    return '<p>%s</p><table border>%s</table>' % (
      OctetString.displayValue(self,valueindex,commandbutton),
      '\n'.join(result_lines)
    )


syntax_registry.registerAttrType(
  LogonHours.oid,[
    '1.2.840.113556.1.4.64', # logonHours
  ]
)


class CountryCode(SelectList):
  oid = 'CountryCode-oid'
  desc = 'Numerical country code'
  attr_value_dict = ldap.cidict.cidict(w2lapp.cnf.countries.countryCode_dict)


syntax_registry.registerAttrType(
  CountryCode.oid,[
    '1.2.840.113556.1.4.25', # countryCode
  ]
)


class InstanceType(BitArrayInteger):
  """
  http://msdn.microsoft.com/library/en-us/adschema/adschema/a_instancetype.asp
  """
  oid = 'InstanceType-oid'
  flag_desc_table = (
    ('The head of naming context.',0x00000001),
    ('This replica is not instantiated.',0x00000002),
    ('The object is writable on this directory.',0x00000004),
    ('The naming context above this one on this directory is held.',0x00000008),
    ('The naming context is in the process of being constructed for the first time via replication.',0x00000010),
    ('The naming context is in the process of being removed from the local DSA.',0x00000020),
  )

syntax_registry.registerAttrType(
  InstanceType.oid,[
    '1.2.840.113556.1.2.1', # instanceType
  ]
)


class DNWithOctetString(DistinguishedName):
  oid = '1.2.840.113556.1.4.903'
  desc = 'DNWithOctetString'

  def displayValue(self,valueindex=0,commandbutton=0):
    octet_tag,count,octet_string,dn = self.attrValue.split(':')
    return ':'.join([
      octet_tag,count,octet_string,
      DisplayDN(self._sid,self._form,self._ls,dn,commandbutton=commandbutton)
    ])


class DNWithString(DNWithOctetString):
  oid = '1.2.840.113556.1.4.904'
  desc = 'DNWithString'


class MicrosoftLargeInteger(Integer):
  oid = '1.2.840.113556.1.4.906'
  desc = 'Integer guaranteed to support 64 bit numbers'


class ObjectSecurityDescriptor(OctetString):
  oid = '1.2.840.113556.1.4.907'
  desc = 'Object-Security-Descriptor'


class Interval(MicrosoftLargeInteger):
  oid = 'Interval-oid'
  desc = 'Large integer with timestamp expressed as 100 nanoseconds since 1601-01-01 00:00'

  def displayValue(self,valueindex=0,commandbutton=0):
    if self.attrValue=='9223372036854775807':
      return '-1: unlimited/off'
    else:
      delta = (long(self.attrValue)-116444736000000000L)/10000000
      if delta>=0:
        return utctime.strftimeiso8601(time.gmtime(delta)).encode('ascii')
      else:
        return self.attrValue.encode('ascii')

# Register certain attribute types for Interval
syntax_registry.registerAttrType(
  Interval.oid,[
    '1.2.840.113556.1.4.159',  # accountExpires
    '1.2.840.113556.1.4.49',   # badPasswordTime
    '1.2.840.113556.1.4.52',   # lastLogon
    '1.2.840.113556.1.4.1696', # lastLogonTimestamp
    '1.2.840.113556.1.4.51',   # lastLogoff
    '1.2.840.113556.1.4.96',   # pwdLastSet
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))



# Work around for Active Directory of Windows 2000:
# Register syntaxes by odd names
syntax_registry.oid2syntax['Boolean'] = Boolean
syntax_registry.oid2syntax['DN'] = DistinguishedName
syntax_registry.oid2syntax['Integer'] = Integer
syntax_registry.oid2syntax['DirectoryString'] = DirectoryString
syntax_registry.oid2syntax['GeneralizedTime'] = GeneralizedTime

syntax_registry.registerAttrType(
  DistinguishedName.oid,[
    'configurationNamingContext',
    'defaultNamingContext',
    'dsServiceName',
    'rootDomainNamingContext',
    'schemaNamingContext',
  ]
)

# MS AD declares these attributes with OctetString
# syntax but Binary syntax is more suitable
syntax_registry.registerAttrType(
  Binary.oid,[
    '1.2.840.113556.1.4.645',    # userCert
    '1.2.840.113556.1.4.138',    # userParameters
  ]
)

syntax_registry.registerAttrType(
  Uri.oid,[
    '1.2.840.113556.1.4.583', # meetingURL
  ]
)
